@@ -21,6 +21,8 @@
 
 #include "guarana/GuaraNative.h"
 
+char * guarana_version = GVER;
+
 /* These are initialized by java_lang_Class_init_function_pointers, in Class.c */
 
 HArrayOfObject* (*p_makeParameters)(Method *meth);
@@ -106,7 +108,12 @@ struct Hjava_lang_Object *guarana_wrap(struct Hjava_lang_Class *clazz, jvalue va
     return execute_java_constructor(0, javaLangCharacterClass, "(C)V", val.i);
   case 'F':
     *size = 1;
-    return execute_java_constructor(0, javaLangFloatClass, "(F)V", val.f);
+    return execute_java_constructor(0, javaLangFloatClass, "(F)V",
+				    *(int*)&val.f);
+    /* FIXME: a float would be promoted to double because of varargs.
+     * For consistency, kaffe should always pass floats as doubles,
+     * but it doesn't; this breaks native methods that take floats as
+     * arguments or return them.  */
   case 'D':
     *size = 2;
     return execute_java_constructor(0, javaLangDoubleClass, "(D)V", val.d);
@@ -409,14 +416,14 @@ guarana_va_list_reify_method_invocation(struct HBR_unicamp_Guarana_MetaObject *m
       break;
     case 'J':
       argl->j = va_arg(args, jlong);
-      ++args;
+      ++argl;
       break;
     case 'F':
       argl->f = va_arg(args, jfloat);
       break;
     case 'D':
       argl->d = va_arg(args, jdouble);
-      ++args;
+      ++argl;
       break;
     case '[':
       while (*sig == '[')
