@@ -38,6 +38,49 @@
   } \
 } while (0)
 
+#define checkExcept(meth_, op_) do if (op_) { \
+  NativeOperation *op = (NativeOperation*)op_; \
+  Hjava_lang_Class *mclass = meth_->class; \
+  constIndex *excepts = meth_->declared_exceptions; \
+  int i, n = meth_->ndeclared_exceptions; \
+  for(i = 0; i < n; ++i) { \
+    static Hjava_lang_Class *javaLangError = 0; \
+    static Hjava_lang_Class *javaLangRuntimeException = 0; \
+    Hjava_lang_Class *xclass = getClass(excepts[i], mclass); \
+    if (javaLangError == 0) \
+      javaLangError = lookupClass("java/lang/Error"); \
+    if (instanceof(javaLangError, xclass)) \
+      continue; \
+    if (javaLangRuntimeException == 0) \
+      javaLangRuntimeException = lookupClass("java/lang/RuntimeException"); \
+    if (instanceof(javaLangRuntimeException, xclass)) \
+      continue; \
+    switch (op->kind & op_kind_mask) { \
+    case op_nop: \
+      break; \
+       \
+    case op_method_invocation: \
+    case op_constructor_invocation: \
+      { \
+	methods *meth = op->op_id.method; \
+	Hjava_lang_Class *decclass = meth->class; \
+	constIndex *excepts = meth->declared_exceptions; \
+	int j, nr = meth->ndeclared_exceptions; \
+	for (j = 0; j < nr; ++j) { \
+	  if (instanceof(getClass(excepts[j], decclass), xclass)) \
+	    break; \
+	} \
+	if (j < nr) \
+	  break; \
+      } \
+      /* do not break */ \
+	 \
+    default: \
+      SignalError("java/lang/IllegalArgumentException", "invalid exception type"); \
+    } \
+  } \
+} while (0)
+
 #define checkMobj(this) do { \
   if (this->obj->meta_object != this->mobj) \
     SignalError("java/lang/IllegalAccessException", "MetaObject changed"); \
@@ -57,18 +100,20 @@ struct HBR_unicamp_Guarana_Operation* BR_unicamp_Guarana_OperationFactoryInterna
   return (struct HBR_unicamp_Guarana_Operation*)op;
 }
 
-struct HBR_unicamp_Guarana_Operation* BR_unicamp_Guarana_OperationFactoryInternal_invoke(struct HBR_unicamp_Guarana_OperationFactoryInternal *this_, struct Hjava_lang_reflect_Method *meth, HArrayOfObject *arguments, struct HBR_unicamp_Guarana_Operation *repl) {
+struct HBR_unicamp_Guarana_Operation* BR_unicamp_Guarana_OperationFactoryInternal_invoke(struct HBR_unicamp_Guarana_OperationFactoryInternal *this_, struct Hjava_lang_reflect_Method *meth_, HArrayOfObject *arguments, struct HBR_unicamp_Guarana_Operation *repl) {
   OperationFactory *this = (OperationFactory*)this_;
   NativeOperation *op;
   jint i, len;
   struct wrapArgVec *argvec;
+  methods *meth = CLASS_METHODS(unhand(meth_)->clazz) + unhand(meth_)->slot;
   
   checkMobj(this);
-  checkCompat(unhand(meth)->returnType, repl);
+  checkCompat(unhand(meth_)->returnType, repl);
   checkTarget(this->obj, repl);
+  checkExcept(meth, repl);
   op = guarana_new_native_operation(this->obj, getCurrentThread(), repl);
   op->kind |= op_method_invocation | op_wrapped;
-  op->op_id.method = CLASS_METHODS(unhand(meth)->clazz) + unhand(meth)->slot;
+  op->op_id.method = meth;
 
   /* FIXME: access array reflexively? */
   len = obj_length(arguments);
@@ -89,13 +134,15 @@ struct HBR_unicamp_Guarana_Operation* BR_unicamp_Guarana_OperationFactoryInterna
   NativeOperation *op;
   int i, len;
   struct wrapArgVec *argvec;
+  methods *meth = CLASS_METHODS(unhand(constr)->clazz) + unhand(constr)->slot;
   
   checkMobj(this);
   checkCompat(&voidClass, repl);
   checkTarget(this->obj, repl);
+  checkExcept(meth, repl);
   op = guarana_new_native_operation(this->obj, getCurrentThread(), repl);
   op->kind |= op_constructor_invocation | op_wrapped;
-  op->op_id.method = CLASS_METHODS(unhand(constr)->clazz) + unhand(constr)->slot;
+  op->op_id.method = meth;
 
   /* FIXME: access array reflexively? */
   len = obj_length(arguments);
