@@ -84,5 +84,5 @@ KAFFE_NATIVE(BR_unicamp_Guarana_Result_getShortValue)
 KAFFE_NATIVE(BR_unicamp_Guarana_Result_getIntValue)
 KAFFE_NATIVE(BR_unicamp_Guarana_Result_getLongValue)
 KAFFE_NATIVE(BR_unicamp_Guarana_Result_getCharValue)
-KAFFE_NATIVE(BR_unicamp_Guarana_Result_getFloatValue0)
+KAFFE_NATIVE(BR_unicamp_Guarana_Result_getFloatValueAsDouble)
 KAFFE_NATIVE(BR_unicamp_Guarana_Result_getDoubleValue)
